:: ----------------------------------------------------------------------
:: Titel:     FotoRenamer
:: Zweck:     Batchdatei um Fotos nach festem Schema umzubenennen
::            Ordnet gleichnamigen Dateien z.B. *.jpg *.tif, *.nef
::            den gleichen Namen zu, welcher aus EXIF-Datum, Text und
::            fortlaufender Nummer besteht (090315_aaa_bbb_###_ccc).
:: Autor:     Sarah Huber und Ren Brgi
:: Erfordert: exifutils 2.8 von hugsan
::            beep.exe
:: Fr:       Microsoft Windows 2000, XP, Vista (Konsole)
:: Bemerkung: Jede Benutzung erfolgt auf eigene Verantwortung
::            Die Autoren lehnen jede Haftung ab!
::            Der Code ist frei und darf frei benutzt/veraendert werden
::
:: Version History:
::         x.x    YYYYMMDD HH   Autor         Beschreibung
:: version=0.10   20090321      Sarah Huber   Konzept, Par: a,c,t,n
:: version=1.00   20090325 17   Ren Brgi    Grundfunktionen, Par: l,s,d,e,i
:: version=1.10   20090326 10   Ren Brgi    Suffix frei definierbar, Par: y
:: version=1.20   20090327 17   Ren Brgi    Datum auch aus Datei, Bug
::
:: ----------------------------------------------------------------------
	
@ECHO OFF

:: ----------------------------------------------------------------------
:: DEFAULT-WERTE (Vom Benutzer anpassbar)
:: ----------------------------------------------------------------------
	
	::	title, vers, versDate:  Nur bei Skriptnderungen neu zu setzen:
SET title=FRen
SET vers=V1.2
SET versDate=27.03.2009
	::	a, c, t: Standardwerte fr Autor, Kamera und Zusatztext
	::  Grundstzlich kann fr a, c, t jeder beliebige Text eingegeben werden. 
	::  a, c, t knnen leer sein, aber keine Leerzeichen enthalten
	::  a, c, t darf nicht einer der Parameter sein (-a, -c, -t, -n...)
	::  Die Zeichenfolge $fn$ in t wird mit dem alten Dateinamen ersetzt
SET a=RB
SET c=300
SET t=
	::	n: Standardwert fr Startnummer (im Normalfall 1)
SET n=1
	::  l: Standardmssig Logfile erstellen? Dann auf 1, sonst auf 0:
SET l=0
	::	s: Sicherheitswert zwischen 0 und 4 (sicher), siehe :freninfo
SET s=3
	::	d: Anzahl Digits des Zhlers: Wert zwischen 1 und 5
SET d=3
	::  exifDateORfileDate:  0: ExifUtils bentzen, 1: Dateidatum bentzen
	::  d.h. sind ExifUtils von Hugsan nicht installiert, dann auf 1 setzen!
SET exifDateORfileDate=0
	::  Ist beep.exe installiert, dann beepOn auf 1 setzten, sonst auf 0:
SET beepOn=1

	:: ext1..ext5: Nach diesen Dateiendungen wird gesucht 
	:: ACHTUNG: ext1 darf NICHT LEER sein
	:: ACHTUNG: Alle Endungen (sofern nicht leer) benoetigen exakt 3 Zeichen
SET ext1=jpg
SET ext2=nef
SET ext3=tif
SET ext4=
SET ext5=
	:: ext_1, ext_2: Diese Dateien werden mit umbenannt, wenn sie ebenfalls existieren
SET ext_1=txt
SET ext_2=

:: ----------------------------------------------------------------------
:: ----------------------------------------------------------------------
:: Ab hier beginnt der Programm-Code
:: ----------------------------------------------------------------------

:: Variablen initialisieren, die nicht veraendert werden sollten
SET i=
SET e=
SET y=
SET fehler=

:: Command-Prozessor-Einstellungen
SETLOCAL ENABLEDELAYEDEXPANSION
SETLOCAL ENABLEEXTENSIONS

:: Fr die Information ber einzulesender Dateitypen
SET list1=*.%ext1%
IF NOT "%ext2%"=="" SET list1=%list1%, *.%ext2%
IF NOT "%ext3%"=="" SET list1=%list1%, *.%ext3%
IF NOT "%ext4%"=="" SET list1=%list1%, *.%ext4%
IF NOT "%ext5%"=="" SET list1=%list1%, *.%ext5%
IF NOT "%ext_1%"=="" SET list2=*.%ext_1%
IF NOT "%ext_2%"=="" SET list2=%list2%, *.%ext_2%,

:: Fall, dass Ordnerangabe fehlt
SET folder=%1
if [%folder%] == [] (
	call:freninfo
	ECHO     Es fehlt die Ordnerangabe
	GOTO Done
)

:: Fall, dass Ordnerangabe nicht gueltig
SET folder=%folder:"=%
SET eoffolder=%folder:~-1%
if [%eoffolder%] == [\] SET folder=%folder:~0,-1%
	echo %folder%
if not exist "%folder%\" (
	call:freninfo
	ECHO     Der angegebene Ordner ist nicht gueltig: %1
	GOTO Done
)

:: Fall, dass Ordnerangabe gueltig
SET invfolder=%folder:\=/%
SET SearchList="%folder%\*.%ext1%"
IF NOT "%ext2%"=="" SET SearchList=%SearchList% "%folder%\*.%ext2%"
IF NOT "%ext3%"=="" SET SearchList=%SearchList% "%folder%\*.%ext3%"
IF NOT "%ext4%"=="" SET SearchList=%SearchList% "%folder%\*.%ext4%"
IF NOT "%ext5%"=="" SET SearchList=%SearchList% "%folder%\*.%ext5%" 

:: Parameter einlesen
SET dd=
SET reada=
SET readc=
SET readt=
SET readn=
SET readi=
SET readl=
SET reads=
SET readd=
SET reade=
SET ready=
	:: erste 8 Parameter einlesen
FOR %%a IN (%2 %3 %4 %5 %6 %7 %8 %9) DO (
	if !reada!==1 SET a=%%a && SET reada=
	if !readc!==1 SET c=%%a && SET readc=
	if !readt!==1 SET t=%%a && SET readt=
	if !readn!==1 SET n=%%a && SET readn=
	if !reads!==1 SET s=%%a && SET reads=
	if !readd!==1 SET dd=%%a && SET readd=
	if [%%a]==[-A] SET reada=1&& SET a=
	if [%%a]==[-C] SET readc=1&& SET c=
	if [%%a]==[-T] SET readt=1&& SET t=
	if [%%a]==[-N] SET readn=1
	if [%%a]==[-I] SET readi=1
	if [%%a]==[-L] SET readl=1
	if [%%a]==[-S] SET reads=1
	if [%%a]==[-D] SET readd=1
	if [%%a]==[-E] SET reade=1
	if [%%a]==[-Y] SET ready=1
	if [%%a]==[-a] SET reada=1&& SET a=
	if [%%a]==[-c] SET readc=1&& SET c=
	if [%%a]==[-t] SET readt=1&& SET t=
	if [%%a]==[-n] SET readn=1
	if [%%a]==[-i] SET readi=1
	if [%%a]==[-l] SET readl=1
	if [%%a]==[-s] SET reads=1
	if [%%a]==[-d] SET readd=1
	if [%%a]==[-e] SET reade=1
	if [%%a]==[-y] SET ready=1
)
	:: letzter Parameter wiederholen und weitere 8 einlesen
SHIFT && SHIFT && SHIFT && SHIFT && SHIFT && SHIFT && SHIFT && SHIFT
FOR %%a IN (%1 %2 %3 %4 %5 %6 %7 %8 %9) DO (
	if !reada!==1 SET a=%%a && SET reada=
	if !readc!==1 SET c=%%a && SET readc=
	if !readt!==1 SET t=%%a && SET readt=
	if !readn!==1 SET n=%%a && SET readn=
	if !reads!==1 SET s=%%a && SET reads=
	if !readd!==1 SET dd=%%a && SET readd=
	if [%%a]==[-A] SET reada=1&& SET a=
	if [%%a]==[-C] SET readc=1&& SET c=
	if [%%a]==[-T] SET readt=1&& SET t=
	if [%%a]==[-N] SET readn=1
	if [%%a]==[-I] SET readi=1
	if [%%a]==[-L] SET readl=1
	if [%%a]==[-S] SET reads=1
	if [%%a]==[-D] SET readd=1
	if [%%a]==[-E] SET reade=1
	if [%%a]==[-Y] SET ready=1
	if [%%a]==[-a] SET reada=1&& SET a=
	if [%%a]==[-c] SET readc=1&& SET c=
	if [%%a]==[-t] SET readt=1&& SET t=
	if [%%a]==[-n] SET readn=1
	if [%%a]==[-i] SET readi=1
	if [%%a]==[-l] SET readl=1
	if [%%a]==[-s] SET reads=1
	if [%%a]==[-d] SET readd=1
	if [%%a]==[-e] SET reade=1
	if [%%a]==[-y] SET ready=1
)
	
:: Space abtrennen
IF NOT "%a%"=="" set a=%a: =%
IF NOT "%c%"=="" set c=%c: =%
IF NOT "%t%"=="" set t=%t: =%
IF NOT "%n%"=="" set n=%n: =%
IF NOT "%s%"=="" set s=%s: =%
IF NOT "%e%"=="" set e=%e: =%
IF NOT "%dd%"=="" set dd=%dd: =%
IF "%dd%"=="1" SET d=%dd%
IF "%dd%"=="2" SET d=%dd%
IF "%dd%"=="3" SET d=%dd%
IF "%dd%"=="4" SET d=%dd%
IF "%dd%"=="5" SET d=%dd%

:: Textbeinhaltende Variable: Wenn nachher nichts folgt, dann sind sie "leer"
FOR %%a IN ("-A", "-a", "-C", "-c", "-T", "-t", "-N", "-n", "-I", "-i", "-L", "-l", "-S", "-s", "-D", "-d", "-E", "-e", "-Y", "-y") DO (
	IF "%a%"==%%a SET a=
	IF "%c%"==%%a SET c=
	IF "%t%"==%%a SET t=
)
IF NOT "%a%"=="" SET a=_%a%
IF NOT "%c%"=="" SET c=_%c%
IF NOT "%t%"=="" SET t=_%t%

:: Sicherheits-Stufen
SET vorschau=1
SET fragen=1
SET umbenennen=1
::0=nie 1=immer 2=wenn keine Warnung
IF "%s%"=="4" SET vorschau=1&& SET fragen=0&& SET umbenennen=0
IF "%s%"=="3" SET vorschau=1&& SET fragen=1&& SET umbenennen=2
IF "%s%"=="2" SET vorschau=1&& SET fragen=2&& SET umbenennen=2
IF "%s%"=="1" SET vorschau=1&& SET fragen=0&& SET umbenennen=2
IF "%s%"=="0" SET vorschau=0&& SET fragen=0&& SET umbenennen=1

:: Log-Datei anlegen
ECHO.
ECHO  FotoRenamer %title% %vers%
IF [%readl%]==[1] ECHO Log-Datei zu %title% %vers% vom %DATE%,%TIME%> "%folder%\log.txt"
)

:: TESTEN UND AUSFHREN
SET ausfuehren=1
IF "%vorschau%"=="1" call:CheckFolder ausfuehren
IF "%umbenennen%"=="1" call:BatchRename 
IF "%umbenennen%"=="2" IF "%ausfuehren%"=="1" call:BatchRename
GOTO Done


:FileExist
	:: Keine Dateien gefunden
	SET fehler=Dateikollision
	SET	umbenennen=0
	GOTO Abbruch

:NoFile
	:: Keine Dateien gefunden
	SET fehler=leerem Ordner
	ECHO.
	ECHO  Im Ordner %folder% sind keine der folgenden Dateien vorhanden:
	ECHO  %list1%
	ECHO.
	IF [%readl%]==[1] ( 
		ECHO. >> "%folder%\log.txt"
		ECHO  Im Ordner %folder% sind keine der folgenden Dateien vorhanden: >> "%folder%\log.txt"
		ECHO  %list1%
		ECHO. >> "%folder%\log.txt"
	)
	SET	umbenennen=0
	GOTO Abbruch

:FileMissing
	:: bentigte Datei fehlt
	SET fehler=fehlender Datei exiflist.exe
	ECHO.
	ECHO Eine der benoetigten Dateien fehlt:
	ECHO exiflist
	IF [%readl%]==[1] 
		ECHO. >> "%folder%\log.txt"
		ECHO Eine der benoetigten Dateien fehlt (exiflist.exe): >> "%folder%\log.txt"
	)
	GOTO Abbruch

:Abbruch
	ECHO.
	IF "%fehler%"=="Benutzer" (
		ECHO  Ausfuehrung durch %fehler% abgebrochen!
	) Else (
		ECHO  Ausfuehrung wegen %fehler% abgebrochen!
		IF "%beepOn%"=="1" BEEP 700 600
	)
	ECHO.
	IF [%readl%]==[1] (
		ECHO. >> "%folder%\log.txt"
		ECHO  Ausfuehrung wegen %fehler% abgebrochen! >> "%folder%\log.txt"
		ECHO. >> "%folder%\log.txt"
	)
	ECHO.
	ECHO ----------------------------------------------------------------------
	ECHO.
	IF [%reade%]==[1] EXIT
	Exit /b
	
:Done	
	:: Ende der Applikation
	IF "%fehler%"=="" (
		ECHO.
		ECHO ----------------------------------------------------------------------
		ECHO.
	)
	IF [%reade%]==[1] EXIT
	EXIT /b
	



:: ---------------------------------------------------------------------
:: FUNKTIONEN
:: ---------------------------------------------------------------------
:MakeAbsolute file base -- makes a file name absolute considering a base path
::                      -- file [in,out] - variable with file name to be converted, or file name itself for result in stdout
::                      -- base [in,opt] - base path, leave blank for current directory
::$created 20060101 :$changed 20080219 :$categories Path
::$source http://www.dostips.com
@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION
set "src=%~1"
if NOT %1=="" set "src=!%~1!"
set "bas=%~2"
if [bas]==[] set "bas=%cd%"
for /f "tokens=*" %%a in ("%bas%\%src%") do set "src=%%~fa"
( ENDLOCAL & REM RETURN VALUES
    IF NOT %1=="" (SET %~1=%src%) ELSE ECHO.%src%
)
GOTO:EOF

:GetExifDate

	:: EXIF-Datum auslesen
	IF NOT [%readi%]==[1] exiflist /q /o l /u /f date-taken "%invfolder%/%fotoname%" > "%temp%\temp3.txt"
	IF [errorlevel]==[1] GOTO FileMissing
	IF [%readi%]==[1] exiflist /q /o l /u /f ip-date "%invfolder%/%fotoname%" > "%temp%\temp3.txt"
	IF [errorlevel]==[1] GOTO FileMissing
	SET completeDateTime=
	SET /P completeDateTime=<"%temp%\temp3.txt"

	:: EXIF-Datum validieren
	IF [!completeDateTime!]==[] SET completeDateTime=_
	SET c1=!completeDateTime:~0,1!
	SET c2=!completeDateTime:~1,1!
	SET c3=!completeDateTime:~2,1!
	SET c4=!completeDateTime:~3,1!
	SET c5=!completeDateTime:~5,1!
	SET c6=!completeDateTime:~6,1!
	SET c7=!completeDateTime:~8,1!
	SET c8=!completeDateTime:~9,1!
	for %%b IN (!c1! !c2! !c3! !c4! !c5! !c6! !c7! !c8!) DO (
		IF NOT %%b==1 IF NOT %%b==2 IF NOT %%b==3 IF NOT %%b==4 IF NOT %%b==5 IF NOT %%b==6 IF NOT %%b==7 IF NOT %%b==8 IF NOT %%b==9 IF NOT %%b==0 SET completeDateTime=
	)
	
	:: Datum entschluesseln
	IF NOT [!completeDateTime!]==[] SET completeDate=!completeDateTime!
	IF [!completeDateTime!]==[] SET dateWarning2=1
	SET year=!completeDate:~2,2!
	IF [%ready%]==[1] SET year=!completeDate:~0,4!
	SET month=!completeDate:~5,2!
	SET day=!completeDate:~8,2!

GOTO:EOF

:GetFileDate

::  Daten: /TC (creation)  /TA (last access)  /TW (last written)
	Dir /TW "%folder%\%fotoname%" > "%temp%\temp3.txt"
	SET /a i=1
	
	for /f "tokens=1,* delims=] " %%a in ('FIND /I "." "%temp%\temp3.txt"') do (
		IF !i!==2 SET completeDateTime=%%a
		SET /a i=!i!+1
	)

	:: Datum entschluesseln
	IF NOT [!completeDateTime!]==[] SET completeDate=!completeDateTime!
	IF [!completeDateTime!]==[] SET dateWarning2=1
	SET year=!completeDate:~8,2!
	IF [%ready%]==[1] SET year=!completeDate:~6,4!
	SET month=!completeDate:~3,2!
	SET day=!completeDate:~0,2!

GOTO:EOF

:freninfo Dateiinformationen einblenden
	ECHO.
	ECHO FotoRenamer %title% %vers% (%versDate%) - Fotos nach festem Schema umbenennen
	ECHO.
	ECHO   Syntax:  
	ECHO.
	ECHO   FREN folder [-option] [-option] [-option]
	ECHO.
	ECHO   folder     : Relative oder absolute Verzeichnisangabe
	ECHO   [-a aut]   : Bildautor  (freier Text, Default: %a%)
	ECHO   [-c cam]   : Kamera-Nr  (freier Text, Default: %c%)
	ECHO   [-t txt]   : Zusatztext (freier Text, Default: %t%)
	ECHO   [-n ###]   : StartNr    (Default: %n%)
	ECHO   [-d #]     : Anz. Digit (Default: %d%, moeglich: 1-5)
	ECHO   [-i]       : IP-Date statt EXIF-Date-Taken verwenden
	ECHO   [-l]       : Logfile anlegen
	ECHO   [-e]       : Nach dem Bearbeiten Konsole schliessen (Exit)
	ECHO   [-y]       : Jahresangabe im Datum 4stellig statt 2stellig
	ECHO   [-s #]     : Sicherheitseinstellung (Default: %s%)
	ECHO                 0: Umbenennen unmittelbar ausfuehren
	ECHO                 1: Wenn keine Warnung, nach Testlauf autom. umbenennen
	ECHO                 2: Vor Umbenennen nur bei Warnhinweis quittieren
	ECHO                 3: Vor Umbenennen immer quittieren
	ECHO                 4: Nur Testlauf ausfuehren
	ECHO   Die Reihenfolge der Optionen ist nicht wesentlich
	ECHO.
	ECHO   Neuer Dateiname:  091231_aut_cam_###_txt
	ECHO.
	ECHO.  Bemerkungen:
	ECHO.  - Leerzeichen fuer Texte in Optionen (-a,-c,-t) sind nicht zulaessig
	ECHO.  - Zeichenfolge $fn$ im Zusatztext wird mit altem Dateinamen ersetzt
	ECHO.  - Verzeichnisse mit Leerzeichen muessen zwischen "" gesetzt werden
	ECHO.  - Angezeigt wird: Alter Dateiname - Neuer Dateiname - Datum
	ECHO.  - Nach den folgenden Dateien wird gesucht (im Skript aenderbar):
	ECHO.    %list1%
	ECHO.  - Gleichnamige Dateien der folgenden Typen werden mit umbenannt:
	ECHO.    %list2%
	ECHO.  - Bei mehreren Bildern mit gleichem Dateinamen wird nur das Datum
	ECHO.    der ersten Datei geprueft und fuer alle weiteren uebernommen
	ECHO.  - Bei fehlendem EXIF-Datum / bei Zeitsprung wird Warnung ausgegeben
	ECHO.    (z.B. wenn Bildzaehler der Kamera waehrend Fotos auf 0 sprang)
	ECHO.  - Fehlt das Datum, so wird das Datum der letzten Datei verwendet
	ECHO.  - Der Bildzaehler wird fuer jeden neuen Tag auf StartNr gesetzt
	ECHO.  - Dateikollisionen zwischen den neuen Dateinamen werden nicht erkannt
	ECHO.  - Bei der ersten Dateikollision erfolgt Abbruch 
	ECHO.  - Es werden keine Unterverzeichnisse bearbeitet
	ECHO.  - Default-Werte koennen im obersten Teil des Skriptes gesetzt werden
	ECHO.  - Je nachdem ob ExifUtils (Hugsan) oder Beep.exe vorhanden sind,
	ECHO.    muss im obersten Teil des Skriptes das Flag gesetzt werden!
	ECHO.  - Aufrufe aus Batchdateien z.B. wie folgt:  CALL FREN folder -e -s 1
	ECHO   - Jegliche Benutzung erfolgt auf eigene Verantwortung!
	ECHO.
	ECHO.
	ECHO.---------------------------------------------------------------------
	ECHO.
GOTO:EOF

:CheckFolder Testdurchlauf ausfuehren

:: Ordner ausgeben
SET absfolder=%folder%
IF NOT [%folder:~1,1%] == [:] call:MakeAbsolute absfolder "."
ECHO.
ECHO TESTLAUF - Verzeichnis wird geprueft:
ECHO.
ECHO   %absfolder%
ECHO.
IF [%readl%]==[1] (
	ECHO. >> "%folder%\log.txt"
	ECHO TESTLAUF - Verzeichnis wird geprueft: >> "%folder%\log.txt"
	ECHO. >> "%folder%\log.txt"
	ECHO   %absfolder% >> "%folder%\log.txt"
	ECHO. >> "%folder%\log.txt"
)

::Dateien auflisten und FOR-Schlaufe vorbereiten
Dir %SearchList% /B /ON > "%temp%\temp.txt"
SET lastentry=
IF EXIST "%temp%\temp2.txt" del "%temp%\temp2.txt"
set noFile=1
::wait 1
for /f "skip=2 tokens=1,* delims=]"  %%a in ('FIND /V /N /I "" "%temp%\temp.txt"') do (
	::call var set fotoname=name of %%b
	SET fotoname=%%b
	SET fotoname=!fotoname:~0,-4!
	IF /I NOT [!fotoname!]==[!lastentry!] ECHO %%b>> "%temp%\temp2.txt"
	SET lastentry=!fotoname!
	set noFile=0
)

IF %noFile%==1 GOTO NoFile

SET dateWarning1=
SET dateWarning2=
SET existWarning=
SET anyWarning=
SET lastdate=1
SET /a fNr=%n%
SET zaehler=0
SET zaehlerFiles=0
SET zaehlerExt1=0
SET zaehlerExt2=0
SET zaehlerExt3=0
SET zaehlerExt4=0
SET zaehlerExt5=0
SET zaehlerExt_1=0
SET zaehlerExt_2=0
SET completeDate=2000:01:01

:: Fr jede Datei einzeln ausfhren
::wait 1
for /f "skip=2 tokens=1,* delims=]" %%a in ('FIND /V /N /I "" "%temp%\temp2.txt"') do (
	SET fotoname=%%b
	SET /a zaehler=!zaehler!+1
	
	:: EXIF-Datum auslesen
	IF "%exifDateORfileDate%"=="0" Call:GetExifDate
	IF NOT "%exifDateORfileDate%"=="0" Call:GetFileDate
	SET /a newdate=1!year!!month!!day!
	SET fotoname=!fotoname:~0,-4!

	:: Warnung, wenn bei sortiert hherer Datei lteres Datum gefunden
	if [!newdate!] LSS [!lastdate!] SET /a FNr=%n% && ECHO  ----------------------------------------------------------------- && ECHO  I    Achtung:  Datum wechselt nach frueher                      I && ECHO  ----------------------------------------------------------------- && SET dateWarning1=1
	if [!newdate!] GTR [!lastdate!] IF NOT [!lastdate!]==[1] SET /a FNr=%n% && ECHO  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	IF [%readl%]==[1] if [!newdate!] LSS [!lastdate!] ECHO  ----------------------------------------------------------------- >> %folder%\log.txt 
	IF [%readl%]==[1] if [!newdate!] LSS [!lastdate!] ECHO  I    Achtung:  Datum wechselt nach frueher                      I >> %folder%\log.txt && ECHO  ----------------------------------------------------------------- >> %folder%\log.txt
	IF [%readl%]==[1] if [!newdate!] GTR [!lastdate!] IF NOT [!lastdate!]==[1] ECHO  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ >> %folder%\log.txt
	
	:: Neuer Fotoname berechnen
	IF NOT "%t%"=="" SET t=%t:$fn$=!fotoname!%
	IF "%d%"=="1" SET fNrStr=!fNr!
	IF "%d%"=="2" SET fNrStr=00!fNr!&& SET fNrStr=!fNrStr:~-2!
	IF "%d%"=="3" SET fNrStr=000!fNr!&&	SET fNrStr=!fNrStr:~-3!
	IF "%d%"=="4" SET fNrStr=0000!fNr!&& SET fNrStr=!fNrStr:~-4!
	IF "%d%"=="5" SET fNrStr=00000!fNr!&& SET fNrStr=!fNrStr:~-5!
	SET fotoneu=!year!!month!!day!%a%%c%_!fNrStr!!t!

	:: Neuer Fotoname ausgeben
	IF NOT "%ext1%"=="" IF EXIST "%folder%\!fotoname!.%ext1%" ECHO   !fotoname!.%ext1%  -  !fotoneu!.%ext1%  -  !completeDateTime!&&SET /a zaehlerExt1=!zaehlerExt1!+1
	IF NOT "%ext2%"=="" IF EXIST "%folder%\!fotoname!.%ext2%" ECHO   !fotoname!.%ext2%  -  !fotoneu!.%ext2%  -  !completeDateTime!&&SET /a zaehlerExt2=!zaehlerExt2!+1
	IF NOT "%ext3%"=="" IF EXIST "%folder%\!fotoname!.%ext3%" ECHO   !fotoname!.%ext3%  -  !fotoneu!.%ext3%  -  !completeDateTime!&&SET /a zaehlerExt3=!zaehlerExt3!+1
	IF NOT "%ext4%"=="" IF EXIST "%folder%\!fotoname!.%ext4%" ECHO   !fotoname!.%ext4%  -  !fotoneu!.%ext4%  -  !completeDateTime!&&SET /a zaehlerExt4=!zaehlerExt4!+1
	IF NOT "%ext5%"=="" IF EXIST "%folder%\!fotoname!.%ext5%" ECHO   !fotoname!.%ext5%  -  !fotoneu!.%ext5%  -  !completeDateTime!&&SET /a zaehlerExt5=!zaehlerExt5!+1
	IF NOT "%ext_1%"=="" IF EXIST "%folder%\!fotoname!.%ext_1%" ECHO   !fotoname!.%ext_1%  -  !fotoneu!.%ext_1%  -  !completeDateTime!&&SET /a zaehlerExt_1=!zaehlerExt_1!+1
	IF NOT "%ext_2%"=="" IF EXIST "%folder%\!fotoname!.%ext_2%" ECHO   !fotoname!.%ext_2%  -  !fotoneu!.%ext_2%  -  !completeDateTime!&&SET /a zaehlerExt_2=!zaehlerExt_2!+1
	
	IF NOT "%ext1%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext1%" ECHO   !fotoname!.%ext1%  -  !fotoneu!.%ext1%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext2%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext2%" ECHO   !fotoname!.%ext2%  -  !fotoneu!.%ext2%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext3%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext3%" ECHO   !fotoname!.%ext3%  -  !fotoneu!.%ext3%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext4%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext4%" ECHO   !fotoname!.%ext4%  -  !fotoneu!.%ext4%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext5%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext5%" ECHO   !fotoname!.%ext5%  -  !fotoneu!.%ext5%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext_1%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext_1%" ECHO   !fotoname!.%ext_1%  -  !fotoneu!.%ext_1%  -  !completeDateTime!>> %folder%\log.txt
	IF NOT "%ext_2%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoname!.%ext_2%" ECHO   !fotoname!.%ext_2%  -  !fotoneu!.%ext_2%  -  !completeDateTime!>> %folder%\log.txt
	
	IF NOT "%ext1%"=="" IF EXIST "%folder%\!fotoname!.%ext1%" IF EXIST "%folder%\!fotoneu!.%ext1%" SET existWarning=1
	IF NOT "%ext2%"=="" IF EXIST "%folder%\!fotoname!.%ext2%" IF EXIST "%folder%\!fotoneu!.%ext2%" SET existWarning=1
	IF NOT "%ext3%"=="" IF EXIST "%folder%\!fotoname!.%ext3%" IF EXIST "%folder%\!fotoneu!.%ext3%" SET existWarning=1
	IF NOT "%ext4%"=="" IF EXIST "%folder%\!fotoname!.%ext4%" IF EXIST "%folder%\!fotoneu!.%ext4%" SET existWarning=1
	IF NOT "%ext5%"=="" IF EXIST "%folder%\!fotoname!.%ext5%" IF EXIST "%folder%\!fotoneu!.%ext5%" SET existWarning=1
	IF NOT "%ext_1%"=="" IF EXIST "%folder%\!fotoname!.%ext_1%" IF EXIST "%folder%\!fotoneu!.%ext_1%" SET existWarning=1
	IF NOT "%ext_2%"=="" IF EXIST "%folder%\!fotoname!.%ext_2%" IF EXIST "%folder%\!fotoneu!.%ext_2%" SET existWarning=1
	
	:: Variable updaten und Schleife beenden
	IF NOT [!completeDate!]==[] SET lastdate=!newdate!
	SET /a fNr=!fNr!+1
)

:: Information ausgeben
SET /a zaehlerFiles=%zaehlerExt1%+%zaehlerExt2%+%zaehlerExt3%+%zaehlerExt4%+%zaehlerExt5%+%zaehlerExt_1%+%zaehlerExt_2%
SET zaehlerAusgabeDet=%zaehlerExt1% %ext1%
IF NOT "%ext2%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt2% %ext2%
IF NOT "%ext3%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt3% %ext3%
IF NOT "%ext4%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt4% %ext4%
IF NOT "%ext5%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt5% %ext5%
IF NOT "%ext_1%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt_1% %ext_1%
IF NOT "%ext_2%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt_2% %ext_2%
ECHO.
ECHO  %zaehler% Bilder zu bearbeiten
ECHO  %zaehlerFiles% Dateien (%zaehlerAusgabeDet%)
ECHO.
IF [%readl%]==[1] (
	ECHO. >> "%folder%\log.txt"
	ECHO  "%zaehler% Bilder zu bearbeiten" >> "%folder%\log.txt"
	ECHO  "%zaehlerFiles% Dateien (%zaehlerAusgabeDet%" >> "%folder%\log.txt"
	ECHO. >> "%folder%\log.txt"
)

:: Warnungen ausgeben
IF [!dateWarning1!]==[1] ECHO WARNUNG: Es ist ein unerwarteter Zeitsprung aufgetreten && SET anyWarning=1
IF [!dateWarning1!]==[1] ECHO          Dies kann waehrend dem Umbenennen zu Dateikollisionen fuehren!
IF [!dateWarning2!]==[1] ECHO WARNUNG: Es enthalten nicht alle Dateien ein Datum in den EXIF-Feldern && SET anyWarning=1
IF [!dateWarning2!]==[1] ECHO          Die Dateireihenfolge ist nach dem Umbenennen u.U. falsch)
IF [!existWarning!]==[1] GOTO FileExist
IF [!anyWarning!]==[1] 	IF "%beepOn%"=="1" BEEP 700 600
IF [%readl%]==[1] IF [!dateWarning1!]==[1] ECHO WARNUNG: Es ist ein unerwarteter Zeitsprung aufgetreten >> %folder%\log.txt
IF [%readl%]==[1] IF [!dateWarning2!]==[1] ECHO WARNUNG: Es enthalten nicht alle Dateien ein Datum in den EXIF-Feldern >> %folder%\log.txt

:: Dialog (Weiter JA/NEIN)
SET dialog=0
IF [!anyWarning!]==[1] SET code=0 ELSE SET code=1
IF [!code!]==[0] SET fehler=Warnungen
IF [%fragen%]==[0] SET dialog=0
IF [%fragen%]==[1] SET dialog=1
IF [%fragen%]==[2] SET dialog=0
IF [%fragen%]==[2] IF [!anyWarning!]==[1] SET dialog=1
IF [!dialog!]==[1] (
	ECHO. && ECHO.
	IF [%readl%]==[1] ECHO. >> "%folder%\log.txt"
	SET code=0
	SET key=N
	SET /P key="Soll die Umbenennung ausgefuehrt werden? {J/N}  :"
	IF "!key!"=="Y" SET code=1
	IF "!key!"=="y" SET code=1
	IF "!key!"=="J" SET code=1
	IF "!key!"=="j" SET code=1
	IF [!code!]==[0] SET fehler=Benutzer
	IF [%readl%]==[1] IF "!code!"=="1" ECHO  Soll die Umbenennung ausgefuehrt werden?  JA >> %folder%\log.txt 
	IF [%readl%]==[1] IF "!code!"=="0" ECHO  Soll die Umbenennung ausgefuehrt werden?  NEIN >> %folder%\log.txt 
)
ECHO.
IF [%readl%]==[1] ECHO. >> %folder%\log.txt 
SET %~1=!code!
IF [%fehler%]==[Benutzer] GOTO Abbruch

GOTO:EOF


:BatchRename Hauptdurchlauf ausfhren (Umbenennen)

:: Ordner ausgeben
SET absfolder=%folder%
IF NOT [%folder:~1,1%] == [:] call:MakeAbsolute absfolder "."
ECHO.
ECHO UMBENENNEN - Verzeichnis wird bearbeitet:
ECHO.
ECHO   %absfolder% 
ECHO.
IF [%readl%]==[1] (
	ECHO. >> "%folder%\log.txt"
	ECHO UMBENENNEN - Verzeichnis wird bearbeitet: >> "%folder%\log.txt"
	ECHO. >> "%folder%\log.txt"
	ECHO   %absfolder% >> "%folder%\log.txt"
	ECHO. >> "%folder%\log.txt"
)
	
::Dateien auflisten und FOR-Schlaufe vorbereiten
Dir %SearchList% /B /ON > "%temp%\temp.txt"
SET lastentry=
set noFile=1
IF EXIST "%temp%\temp2.txt" del "%temp%\temp2.txt"
::wait 1
for /f "skip=2 tokens=1,* delims=]" %%a in ('FIND /V /N /I "" "%temp%\temp.txt"') do (
	SET fotoname=%%b
	SET fotoname=!fotoname:~0,-4!
	IF /I NOT [!fotoname!]==[!lastentry!] ECHO %%b>> "%temp%\temp2.txt"
	SET lastentry=!fotoname!
	set noFile=0
)
IF %noFile%==1 GOTO NoFile
	
:: Variablen der FOR-Schleife initialisieren
SET lastdate=1
SET fNr=%n%
SET zaehler=0
SET zaehlerFiles=0
SET zaehlerExt1=0
SET zaehlerExt2=0
SET zaehlerExt3=0
SET zaehlerExt4=0
SET zaehlerExt5=0
SET zaehlerExt_1=0
SET zaehlerExt_2=0
SET completeDate=2000:01:01

:: Fr jede Datei einzeln ausfhren
::wait 1
for /f "skip=2 tokens=1,* delims=]" %%a in ('FIND /V /N /I "" "%temp%\temp2.txt"') do (
	SET fotoname=%%b
	SET /a zaehler=!zaehler!+1
	
	:: EXIF-Datum auslesen
	IF "%exifDateORfileDate%"=="0" Call:GetExifDate
	IF NOT "%exifDateORfileDate%"=="0" Call:GetFileDate
	SET /a newdate=1!year!!month!!day!
	SET fotoname=!fotoname:~0,-4!
	
	:: Warnung, wenn bei sortiert hherer Datei lteres Datum gefunden
	if [!newdate!] LSS [!lastdate!] SET /a FNr=%n% && ECHO  ----------------------------------------------------------------- && ECHO  I    Achtung:  Datum wechselt nach frueher                      I && ECHO  ----------------------------------------------------------------- && SET dateWarning1=1
	if [!newdate!] GTR [!lastdate!] IF NOT [!lastdate!]==[1] SET /a FNr=%n% && ECHO  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	IF [%readl%]==[1] if [!newdate!] LSS [!lastdate!] ECHO  ----------------------------------------------------------------- >> %folder%\log.txt 
	IF [%readl%]==[1] if [!newdate!] LSS [!lastdate!] ECHO  I    Achtung:  Datum wechselt nach frueher                      I >> %folder%\log.txt && ECHO  ----------------------------------------------------------------- >> %folder%\log.txt
	IF [%readl%]==[1] if [!newdate!] GTR [!lastdate!] IF NOT [!lastdate!]==[1] ECHO  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ >> %folder%\log.txt

	:: Neuer Fotoname berechnen
	SET t=%t:$filename$=!fotoname!%
	IF "%d%"=="1" SET fNrStr=!fNr!
	IF "%d%"=="2" SET fNrStr=00!fNr!&& SET fNrStr=!fNrStr:~-2!
	IF "%d%"=="3" SET fNrStr=000!fNr!&&	SET fNrStr=!fNrStr:~-3!
	IF "%d%"=="4" SET fNrStr=0000!fNr!&& SET fNrStr=!fNrStr:~-4!
	IF "%d%"=="5" SET fNrStr=00000!fNr!&& SET fNrStr=!fNrStr:~-5!
	SET fotoneu=!year!!month!!day!%a%%c%_!fNrStr!%t%

	:: Dateikollisionen vermeiden --> Abbruch falls existiert
	IF "!fotoname!"=="!fotoneu!" GOTO FileExist
	IF NOT "%ext1%"=="" IF EXIST "%folder%\!fotoneu!.%ext1%" GOTO FileExist
	IF NOT "%ext2%"=="" IF EXIST "%folder%\!fotoneu!.%ext2%" GOTO FileExist
	IF NOT "%ext3%"=="" IF EXIST "%folder%\!fotoneu!.%ext3%" GOTO FileExist
	IF NOT "%ext4%"=="" IF EXIST "%folder%\!fotoneu!.%ext4%" GOTO FileExist
	IF NOT "%ext5%"=="" IF EXIST "%folder%\!fotoneu!.%ext5%" GOTO FileExist
	IF NOT "%ext_1%"=="" IF EXIST "%folder%\!fotoneu!.%ext_1%" GOTO FileExist
	IF NOT "%ext_2%"=="" IF EXIST "%folder%\!fotoneu!.%ext_2%" GOTO FileExist
	
	:: Neuer Fotoname setzen
	IF NOT "%ext1%"=="" IF EXIST "%folder%\!fotoname!.%ext1%" rename "%folder%\!fotoname!.%ext1%"  "!fotoneu!.%ext1%"
	IF NOT "%ext2%"=="" IF EXIST "%folder%\!fotoname!.%ext2%" rename "%folder%\!fotoname!.%ext2%"  "!fotoneu!.%ext2%"
	IF NOT "%ext3%"=="" IF EXIST "%folder%\!fotoname!.%ext3%" rename "%folder%\!fotoname!.%ext3%"  "!fotoneu!.%ext3%"
	IF NOT "%ext4%"=="" IF EXIST "%folder%\!fotoname!.%ext4%" rename "%folder%\!fotoname!.%ext4%"  "!fotoneu!.%ext4%"
	IF NOT "%ext5%"=="" IF EXIST "%folder%\!fotoname!.%ext5%" rename "%folder%\!fotoname!.%ext5%"  "!fotoneu!.%ext5%"
	IF NOT "%ext_1%"=="" IF EXIST "%folder%\!fotoname!.%ext_1%" rename "%folder%\!fotoname!.%ext_1%"  "!fotoneu!.%ext_1%"
	IF NOT "%ext_2%"=="" IF EXIST "%folder%\!fotoname!.%ext_2%" rename "%folder%\!fotoname!.%ext_2%"  "!fotoneu!.%ext_2%"
	
	IF NOT "%ext1%"=="" IF EXIST "%folder%\!fotoneu!.%ext1%" ECHO + !fotoname!.%ext1%  -  !fotoneu!.%ext1%  -  !completeDateTime!&&SET /a zaehlerExt1=!zaehlerExt1!+1
	IF NOT "%ext2%"=="" IF EXIST "%folder%\!fotoneu!.%ext2%" ECHO + !fotoname!.%ext2%  -  !fotoneu!.%ext2%  -  !completeDateTime!&&SET /a zaehlerExt2=!zaehlerExt2!+1
	IF NOT "%ext3%"=="" IF EXIST "%folder%\!fotoneu!.%ext3%" ECHO + !fotoname!.%ext3%  -  !fotoneu!.%ext3%  -  !completeDateTime!&&SET /a zaehlerExt3=!zaehlerExt3!+1
	IF NOT "%ext4%"=="" IF EXIST "%folder%\!fotoneu!.%ext4%" ECHO + !fotoname!.%ext4%  -  !fotoneu!.%ext4%  -  !completeDateTime!&&SET /a zaehlerExt4=!zaehlerExt4!+1
	IF NOT "%ext5%"=="" IF EXIST "%folder%\!fotoneu!.%ext5%" ECHO + !fotoname!.%ext5%  -  !fotoneu!.%ext5%  -  !completeDateTime!&&SET /a zaehlerExt5=!zaehlerExt5!+1
	IF NOT "%ext_1%"=="" IF EXIST "%folder%\!fotoneu!.%ext_1%" ECHO + !fotoname!.%ext_1%  -  !fotoneu!.%ext_1%  -  !completeDateTime!&&SET /a zaehlerExt_1=!zaehlerExt_1!+1
	IF NOT "%ext_2%"=="" IF EXIST "%folder%\!fotoneu!.%ext_2%" ECHO + !fotoname!.%ext_2%  -  !fotoneu!.%ext_2%  -  !completeDateTime!&&SET /a zaehlerExt_2=!zaehlerExt_2!+1
	
	IF NOT "%ext1%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext1%" ECHO !fotoname!.%ext1%  -  !fotoneu!.%ext1%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext2%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext2%" ECHO !fotoname!.%ext2%  -  !fotoneu!.%ext2%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext3%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext3%" ECHO !fotoname!.%ext3%  -  !fotoneu!.%ext3%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext4%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext4%" ECHO !fotoname!.%ext4%  -  !fotoneu!.%ext4%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext5%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext5%" ECHO !fotoname!.%ext5%  -  !fotoneu!.%ext5%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext_1%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext_1%" ECHO !fotoname!.%ext_1%  -  !fotoneu!.%ext_1%  -  !completeDateTime! >> %folder%\log.txt
	IF NOT "%ext_2%"=="" IF [%readl%]==[1] IF EXIST "%folder%\!fotoneu!.%ext_2%" ECHO !fotoname!.%ext_2%  -  !fotoneu!.%ext_2%  -  !completeDateTime! >> %folder%\log.txt

	:: Variable updaten und Schleife beenden
	SET lastdate=!newdate!
	SET /a fNr=!fNr!+1
)  

:: Information ausgeben
SET /a zaehlerFiles=%zaehlerExt1%+%zaehlerExt2%+%zaehlerExt3%+%zaehlerExt4%+%zaehlerExt5%+%zaehlerExt_1%+%zaehlerExt_2%
SET zaehlerAusgabeDet=%zaehlerExt1% %ext1%
IF NOT "%ext2%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt2% %ext2%
IF NOT "%ext3%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt3% %ext3%
IF NOT "%ext4%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt4% %ext4%
IF NOT "%ext5%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt5% %ext5%
IF NOT "%ext_1%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt_1% %ext_1%
IF NOT "%ext_2%"=="" SET zaehlerAusgabeDet=%zaehlerAusgabeDet%, %zaehlerExt_2% %ext_2%
ECHO.
ECHO  %zaehler% Bilder bearbeitet
ECHO  %zaehlerFiles% Dateien (%zaehlerAusgabeDet%)
IF [%readl%]==[1] (
	ECHO. >> "%folder%\log.txt"
	ECHO "%zaehler% Bilder bearbeitet" >> "%folder%\log.txt"
	ECHO "%zaehlerFiles% Dateien (%zaehlerAusgabeDet% >> "%folder%\log.txt"
)
	
GOTO:EOF
:: END OF BATCHRENAME

:: Ende der Funktionen---------------------------------------------------

